# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem_ref unit tests for schema"""

import unittest
from datetime import date

from cubicweb.devtools import PostgresApptestConfiguration
from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_compound.utils import optional_relations, graph_relations
from cubicweb_saem_ref import ConceptSchemeGraph

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


def create_published_records_panel(cnx):
    for name, kind in (
        (u"Mr Pink", u"person"),
        (u"Adams", u"family"),
        (u"Direction de la communication", u"authority"),
    ):
        ar = testutils.authority_record(cnx, name, kind)
        cnx.commit()
        ar.cw_adapt_to('IWorkflowable').fire_transition('publish')
        cnx.commit()


class SchemaConstraintsTC(CubicWebTC):

    configcls = PostgresApptestConfiguration
    assertValidationError = testutils.assertValidationError

    def test_published_constraint_on_contact_point(self):
        """ create two agents: one published P and one not published N.
            create one OU and check that interface will only show P that can become its
            contact point
        """
        with self.admin_access.repo_cnx() as cnx:
            peter = testutils.agent(cnx, u'Peter')
            norton = testutils.agent(cnx, u'Norton')
            cnx.commit()
            norton.cw_adapt_to('IWorkflowable').fire_transition('deprecate')
            cnx.commit()
            ou = testutils.organization_unit(cnx, u'Alice')
            cnx.commit()
            rset = ou.unrelated('contact_point', 'Agent')
            self.assertEqual(rset.one().eid, peter.eid)

    def test_published_constraint_on_archival_agent(self):
        """ create two OU: one published P and one not published N.
            create one Organization and check that interface will only show P that can become its
            archival agent
        """
        with self.admin_access.repo_cnx() as cnx:
            pou = testutils.organization_unit(cnx, u'P OU', archival_roles=['archival'])
            nou = testutils.organization_unit(cnx, u'N OU', archival_roles=['archival'])
            cnx.commit()
            nou.cw_adapt_to('IWorkflowable').fire_transition('deprecate')
            cnx.commit()
            # should be created by testutils.organization_unit() above.
            authority = cnx.find('Organization', name=u'Default authority').one()
            rset = authority.unrelated('archival_unit', 'OrganizationUnit')
            self.assertEqual(rset.one().eid, pou.eid)

    def test_published_constraint_on_use_profile(self):
        with self.admin_access.repo_cnx() as cnx:
            deposit_unit = testutils.organization_unit(cnx, u'AU', archival_roles=['deposit'])
            p1 = testutils.setup_profile(cnx)
            p2 = testutils.setup_profile(cnx)
            cnx.commit()
            p1.cw_adapt_to('IWorkflowable').fire_transition('publish')
            cnx.commit()

            with self.assertValidationError(cnx):
                deposit_unit.cw_set(use_profile=p2)

            deposit_unit.cw_set(use_profile=p1)
            cnx.commit()

    def test_agent_authority_record_is_a_person(self):
        """Authority record for an agent is limited to person"""
        with self.admin_access.repo_cnx() as cnx:
            create_published_records_panel(cnx)
            peter = testutils.agent(cnx, u'Peter')
            self.assertEqual(
                peter.unrelated('authority_record', 'AuthorityRecord').one(),
                cnx.find("AuthorityRecord", has_text=u"Mr Pink").one(),
            )

    def test_organization_unit_authority_record_is_a_authority(self):
        """Authority record for an organization unit is limited to authority"""
        with self.admin_access.repo_cnx() as cnx:
            create_published_records_panel(cnx)
            pou = testutils.organization_unit(cnx, u'P OU')
            self.assertEqual(
                pou.unrelated('authority_record', 'AuthorityRecord').one(),
                cnx.find("AuthorityRecord", has_text=u"Direction de la communication").one(),
            )

    def assertCantChangeRecordKind(self, arecord, kind):
        cnx = arecord._cw
        with self.assertValidationError(cnx) as cm:
            arecord.cw_set(agent_kind=cnx.find('AgentKind', name=kind).one())
        self.assertEqual(cm.exception.errors,
                         {'agent_kind-subject':
                          'This record is used by a relation forbidding to change its type'})

    def test_authority_record_kind_consistency(self):
        with self.admin_access.repo_cnx() as cnx:
            arecord = testutils.authority_record(cnx, u'service', kind=u'authority')
            testutils.organization_unit(cnx, u'unit', authority_record=arecord)
            cnx.commit()

            self.assertEqual(
                [k.name for k in arecord.unrelated('agent_kind', 'AgentKind').entities()],
                [])

            self.assertCantChangeRecordKind(arecord, u'person')

            org = testutils.authority_with_naa(cnx)
            org.cw_set(authority_record=arecord)
            cnx.commit()
            self.assertCantChangeRecordKind(arecord, u'person')

            org.cw_set(authority_record=None)
            self.assertEqual(
                {k.name for k in arecord.unrelated('agent_kind', 'AgentKind').entities()},
                {u'unknown-agent-kind', u'family', u'person'})
            arecord.cw_set(agent_kind=cnx.find('AgentKind', name=u'person').one())
            cnx.commit()

            testutils.agent(cnx, u'bob', authority_record=arecord)
            cnx.commit()
            self.assertCantChangeRecordKind(arecord, u'authority')

    def test_organization_unit_contact_point_in_the_same_authority(self):
        """Create two agents on two distinct authorities. Create an organization unit and check that
        interface will only show consistent proposal for contact point
        """
        with self.admin_access.repo_cnx() as cnx:
            # Create an archival agent in a custom authority
            authority = testutils.authority_with_naa(cnx, name=u'boss club')
            testutils.agent(cnx, u'jdoe', authority=authority)
            # Create another archival agent in the default authority
            norton = testutils.agent(cnx, u'Norton')
            cnx.commit()
            # Now create an organisation unit and check the unrelated list
            ou = testutils.organization_unit(cnx, u'Alice')
            cnx.commit()
            rset = ou.unrelated('contact_point', 'Agent')
            self.assertEqual(rset.one().eid, norton.eid)

    def test_organization_archival_unit_in_whatever_authority(self):
        """Create two organizations on two distinct authorities. Check that interface will show both
        of them as proposal for archival_agent of the default organization
        """
        with self.admin_access.repo_cnx() as cnx:
            # Create an organization unit in a custom authority
            authority = testutils.authority_with_naa(cnx, name=u'boss club')
            testutils.organization_unit(cnx, u'jdoe', archival_roles=['archival'])
            # Create another archival organization unit in the default authority
            testutils.organization_unit(cnx, u'Norton', archival_roles=[u'archival'])
            cnx.commit()
            # Now check the unrelated list for authority's archival_unit
            authority = cnx.find('Organization', name=u'Default authority').one()
            rset = authority.unrelated('archival_unit', 'OrganizationUnit')
            self.assertEqual(len(rset), 2)

    def test_agent_authority_consistency(self):
        with self.admin_access.repo_cnx() as cnx:
            authority1 = testutils.authority_with_naa(cnx)
            authority2 = testutils.authority_with_naa(cnx, name=u'boss club')
            user = self.create_user(cnx, login=u'user', groups=('users',))
            cnx.commit()
            # jdoe is in authority2 and user has no authority: OK
            jdoe = testutils.agent(cnx, u'jdoe', authority=authority2)
            jdoe.cw_set(agent_user=user)
            cnx.commit()
            # jdoe is in authority2 and attempt to set user in authority1: KO
            with self.assertValidationError(cnx):
                user.cw_set(authority=authority1)

            # break link between agent and user
            jdoe.cw_set(agent_user=None)
            cnx.commit()
            user.cw_set(authority=authority1)
            # jdoe is in authority2, user in authority1 and attempt to link them KO
            with self.assertValidationError(cnx):
                jdoe.cw_set(agent_user=user)

    def test_organization_unit_unicity(self):
        with self.admin_access.cnx() as cnx:
            testutils.organization_unit(cnx, u'arch', archival_roles=[u'archival'])
            cnx.commit()

            with self.assertValidationError(cnx):
                testutils.organization_unit(cnx, u'arch', archival_roles=[u'archival'])

            other_authority = testutils.authority_with_naa(cnx, name=u'other authority')
            testutils.organization_unit(cnx, u'arch', archival_roles=[u'archival'],
                                        authority=other_authority)
            cnx.commit()

    def test_agent_unicity(self):
        with self.admin_access.cnx() as cnx:
            testutils.agent(cnx, u'bob')
            cnx.commit()

            with self.assertValidationError(cnx):
                testutils.agent(cnx, u'bob')

            other_authority = testutils.authority_with_naa(cnx, name=u'other authority')
            testutils.agent(cnx, u'bob', authority=other_authority)
            cnx.commit()


class AuthorityRecordTC(CubicWebTC):

    configcls = PostgresApptestConfiguration

    def test_fti(self):
        with self.admin_access.repo_cnx() as cnx:
            agent = testutils.authority_record(cnx, u'Guenievre', kind=u'person',
                                               end_date=date(476, 2, 9))
            address = cnx.create_entity('PostalAddress', street=u"1 av. de l'europe",
                                        postalcode=u'31400', city=u'Toulouse')
            cnx.create_entity('AgentPlace', name=u'place', place_address=address, place_agent=agent)
            cnx.create_entity('AgentFunction', name=u'function', function_agent=agent)
            cnx.create_entity('LegalStatus', term=u'legal status', legal_status_agent=agent)
            cnx.commit()
            for search in (u'guenievre', u'europe', u'place', u'function', u'legal status'):
                with self.subTest(search=search):
                    self.assertEqual(cnx.execute('AuthorityRecord X WHERE X has_text %(search)s',
                                                 {'search': search}).one().eid, agent.eid)


class ConceptSchemeTC(CubicWebTC):

    configcls = PostgresApptestConfiguration

    def test_graph_structure(self):
        graph = ConceptSchemeGraph(self.schema)
        expected = {
            'Concept': {('in_scheme', 'subject'): ['ConceptScheme']},
            'Label': {('label_of', 'subject'): ['Concept']},
        }
        self.assertEqual(graph.parent_structure('ConceptScheme'),
                         expected)

    def test_optional_relations(self):
        graph = ConceptSchemeGraph(self.schema)
        opts = optional_relations(self.schema,
                                  graph.parent_structure('ConceptScheme'))
        expected = {}
        self.assertEqual(opts, expected)

    def test_relations_consistency(self):
        graph = ConceptSchemeGraph(self.schema)
        structure = graph.parent_structure('ConceptScheme')
        structurals, optionals, mandatories = graph_relations(
            self.schema, structure)
        self.assertEqual(structurals - optionals, mandatories)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import sys
from io import BytesIO, StringIO

from cubicweb.devtools import (
    PostgresApptestConfiguration,
    testlib,
)
from cubicweb.cwconfig import CubicWebConfiguration

from cubicweb_eac import ccplugin
from cubicweb_skos import ccplugin as skos

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


class CCPluginTC(testlib.CubicWebTC):

    def setup_database(self):
        super(CCPluginTC, self).setup_database()
        self.orig_config_for = CubicWebConfiguration.config_for
        CubicWebConfiguration.config_for = staticmethod(lambda appid: self.config)

    def tearDown(self):
        CubicWebConfiguration.config_for = self.orig_config_for
        super(CCPluginTC, self).tearDown()


class ImportEacDataCommandTC(CCPluginTC):
    configcls = PostgresApptestConfiguration

    def setup_database(self):
        super(ImportEacDataCommandTC, self).setup_database()
        with self.admin_access.client_cnx() as cnx:
            testutils.authority_with_naa(cnx)  # ensure we have a naming authority
            cnx.commit()

    def run_import_eac(self, *args):
        cmd = [self.appid, self.datapath('D33-100.xml')] + list(args)
        sys.stdout = output = BytesIO()
        try:
            ccplugin.ImportEacData(None).main_run(cmd)
        except SystemExit as exc:
            code = exc.code
        else:
            code = 0
        finally:
            sys.stdout = sys.__stdout__
        return code, output.getvalue()

    def test_ok(self):
        code, output = self.run_import_eac()
        self.assertEqual(code, 0)
        with self.admin_access.repo_cnx() as cnx:
            record = cnx.find('AuthorityRecord').one()
            self.assertEqual(record.dc_title(), u"Centre d'information et d'orientation (CIO)")
            self.assertTrue(testutils.match_ark(record.ark), record.ark)

    def test_bad_authority(self):
        code, output = self.run_import_eac('--authority', 'unexisting')
        self.assertEqual(code, 1)
        self.assertIn('no authority named "unexisting"', output)

    def test_too_many_autorities(self):
        with self.admin_access.client_cnx() as cnx:
            testutils.authority_with_naa(cnx, name=u'hop')
            cnx.commit()
        code, output = self.run_import_eac()
        self.assertEqual(code, 1)
        self.assertIn('there are several authorities', output)

    def test_autority_has_no_naa(self):
        with self.admin_access.client_cnx() as cnx:
            org = testutils.authority_with_naa(cnx)
            org.cw_set(ark_naa=None)
            cnx.commit()
        code, output = self.run_import_eac()
        self.assertEqual(code, 1)
        self.assertIn('not associated to a ARK naming authority', output)


class ImportSkosDataCommandTC(CCPluginTC):

    configcls = PostgresApptestConfiguration

    def run_import_skos(self, *args):
        cmd = [self.appid] + list(args)
        sys.stdout = output = StringIO()
        try:
            skos.ImportSkosData(None).main_run(cmd)
        except SystemExit as exc:
            code = exc.code
        else:
            code = 0
        finally:
            sys.stdout = sys.__stdout__
        return code, output.getvalue()

    def test_lcsv(self):
        with self.admin_access.cnx() as cnx:
            naa = testutils.naa(cnx)
            scheme = cnx.create_entity('ConceptScheme', title=u'lcsv',
                                       ark_naa=naa)
            cnx.commit()
            scheme_uri = scheme.cwuri
            assert scheme.ark_naa
            assert scheme.ark
        code, output = self.run_import_skos(
            self.datapath('lcsv_example_shortened.csv'),
            '--format', 'lcsv',
            '--scheme', scheme.ark,
        )
        self.assertEqual(code, 0, output)
        with self.admin_access.cnx() as cnx:
            scheme = cnx.find('ConceptScheme', cwuri=scheme_uri).one()
            self.assertEqual(len(scheme.reverse_in_scheme), 3)

    def test_lcsv_missing_naa_what(self):
        with self.admin_access.cnx() as cnx:
            scheme = cnx.create_entity('ConceptScheme', title=u'lcsv',
                                       ark=u'0/123', cwuri=u'123')
            cnx.commit()
            assert not scheme.ark_naa
        code, output = self.run_import_skos(
            self.datapath('lcsv_example_shortened.csv'),
            '--format', 'lcsv',
            '--scheme', '0/123',
        )
        self.assertEqual(code, 1, output)
        self.assertIn('command failed: specified concept scheme', output)


if __name__ == '__main__':
    from unittest import main
    main()



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""Functional security tests."""

from datetime import date

from cubicweb.devtools import PostgresApptestConfiguration
from cubicweb.devtools.testlib import CubicWebTC

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


class NonManagerUserTC(CubicWebTC):
    """Tests checking that a user in "users" group only can do things.

    Most of the times, we do not call any assertion method and only rely on no
    error being raised.
    """
    configcls = PostgresApptestConfiguration

    assertUnauthorized = testutils.assertUnauthorized

    login = u'bob'

    def setup_database(self):
        with self.admin_access.cnx() as cnx:
            self.create_user(cnx, self.login, ('users', ))
            authority = testutils.authority_with_naa(cnx)
            cnx.execute('SET U authority O WHERE U login %(login)s, O eid %(o)s',
                        {'login': self.login, 'o': authority.eid})
            cnx.commit()

        self.authority_eid = authority.eid

    def test_create_update_authorityrecord(self):
        with self.admin_access.cnx() as cnx:
            admin_arecord_eid = testutils.authority_record(cnx, name=u'admin').eid
            cnx.commit()

        with self.new_access(self.login).cnx() as cnx:
            arecord = testutils.authority_record(cnx, name=u'a')
            cnx.commit()
            arecord.cw_set(record_id=u'123')
            cnx.commit()

            # can change kind (unless used in constrained relation, but this is tested in
            # unittest_schema)
            arecord.cw_set(agent_kind=cnx.find('AgentKind', name=u'authority').one())
            cnx.commit()

            # can modify record created by someone else
            admin_arecord = cnx.entity_from_eid(admin_arecord_eid)
            admin_arecord.cw_set(record_id=u'123')
            cnx.commit()

        # can create relation and modify those created by someone else
        for rel_etype, rel_from_rtype, rel_to_rtype in [
                ('ChronologicalRelation', 'chronological_predecessor', 'chronological_successor'),
                ('HierarchicalRelation', 'hierarchical_parent', 'hierarchical_child'),
                ('AssociationRelation', 'association_from', 'association_to'),
        ]:
            with self.admin_access.cnx() as cnx:
                # can create / update relation
                relation_eid = cnx.create_entity(
                    rel_etype, **{rel_from_rtype: admin_arecord_eid,
                                  rel_to_rtype: arecord}).eid
                cnx.commit()

            with self.new_access(self.login).cnx() as cnx:
                relation = cnx.entity_from_eid(relation_eid)
                relation.cw_set(
                    start_date=date.today(), **{rel_from_rtype: arecord,
                                                rel_to_rtype: admin_arecord_eid})
                cnx.commit()
                cnx.create_entity(
                    rel_etype, **{rel_from_rtype: admin_arecord_eid,
                                  rel_to_rtype: arecord})
                cnx.commit()

    def test_create_update_sedaprofile(self):
        with self.new_access(self.login).cnx() as cnx:
            profile = testutils.setup_profile(cnx)
            cnx.commit()
            profile.cw_set(user_annotation=u'meh')
            cnx.commit()

            profile.cw_adapt_to('IWorkflowable').fire_transition('publish')
            cnx.commit()
            testutils.setup_profile(cnx, title=u'Clone', new_version_of=profile)
            cnx.commit()

    def test_create_update_vocabulary(self):
        with self.admin_access.cnx() as cnx:
            admin_scheme = testutils.scheme_for_type(cnx, u'seda_keyword_type_to', None,
                                                     u'type1')
            cnx.commit()
            type_concept = admin_scheme.reverse_in_scheme[0]

        with self.new_access(self.login).cnx() as cnx:
            with self.assertUnauthorized(cnx):
                testutils.setup_scheme(cnx, u'my scheme',
                                       u'lab1', u'lab2')

            admin_scheme = cnx.entity_from_eid(admin_scheme.eid)
            type_concept = cnx.entity_from_eid(type_concept.eid)
            with self.assertUnauthorized(cnx):
                admin_scheme.cw_set(title=u'code keyword types')
            with self.assertUnauthorized(cnx):
                admin_scheme.cw_set(code_keyword_type=type_concept)

            admin_scheme.add_concept(u'type2')
            cnx.commit()

            cnx.create_entity('Label', kind=u'alternative', label=u'Le type n.1',
                              label_of=type_concept)
            cnx.commit()

    def test_create_update_agent_in_own_organization(self):
        with self.admin_access.cnx() as cnx:
            other_authority = testutils.authority_with_naa(cnx, name=u'other authority')
            other_agent = testutils.agent(cnx, u'bob', authority=other_authority)
            cnx.commit()
            other_authority_eid = other_authority.eid
            other_agent_eid = other_agent.eid

        with self.new_access(self.login).cnx() as cnx:
            agent = testutils.agent(cnx, u'bob', authority=self.authority_eid)
            cnx.commit()
            agent.cw_set(name=u'bobby')
            cnx.commit()
            agent.cw_delete()
            cnx.commit()
            arecord = testutils.authority_record(cnx, name=u'bobby', kind=u'person')
            agent.cw_set(authority_record=arecord)
            cnx.commit()

            with self.assertUnauthorized(cnx):
                testutils.agent(cnx, u'other bob', authority=other_authority_eid)

            other_agent = cnx.entity_from_eid(other_agent_eid)
            with self.assertUnauthorized(cnx):
                other_agent.cw_set(name=u'bobby')
            with self.assertUnauthorized(cnx):
                other_agent.cw_delete()
            with self.assertUnauthorized(cnx):
                other_arecord = testutils.authority_record(cnx, name=u'other bob', kind=u'person')
                other_agent.cw_set(authority_record=other_arecord)

    def test_create_update_organizationunit_in_own_organization(self):
        with self.admin_access.cnx() as cnx:
            other_authority = testutils.authority_with_naa(cnx, name=u'other authority')
            other_unit = testutils.organization_unit(
                cnx, u'arch', archival_roles=[u'archival'], authority=other_authority)
            profile = testutils.setup_profile(cnx)
            scheme = testutils.scheme_for_type(cnx, u'seda_keyword_type_to', None)
            cnx.commit()
            profile.cw_adapt_to('IWorkflowable').fire_transition('publish')
            cnx.commit()

            other_authority_eid = other_authority.eid
            other_unit_eid = other_unit.eid
            profile_eid = profile.eid
            scheme_eid = scheme.eid

        with self.new_access(self.login).cnx() as cnx:
            roles = (u'archival', u'deposit')
            unit = testutils.organization_unit(
                cnx, u'arch', archival_roles=roles, authority=self.authority_eid)
            cnx.commit()
            unit.cw_set(name=u'archi',
                        use_profile=profile_eid,
                        related_concept_scheme=scheme_eid)
            cnx.commit()
            unit.cw_delete()
            cnx.commit()
            arecord = testutils.authority_record(cnx, name=u'arch', kind=u'authority')
            unit.cw_set(authority_record=arecord)
            cnx.commit()

            with self.assertUnauthorized(cnx):
                testutils.organization_unit(
                    cnx, u'other arch', archival_roles=roles, authority=other_authority_eid)

            other_unit = cnx.entity_from_eid(other_unit_eid)
            with self.assertUnauthorized(cnx):
                other_unit.cw_set(name=u'archi')
            with self.assertUnauthorized(cnx):
                other_unit.cw_set(use_profile=profile_eid)
            with self.assertUnauthorized(cnx):
                other_unit.cw_set(related_concept_scheme=scheme_eid)
            with self.assertUnauthorized(cnx):
                other_unit.cw_delete()
            with self.assertUnauthorized(cnx):
                other_arecord = testutils.authority_record(cnx, name=u'other arch',
                                                           kind=u'authority')
                other_unit.cw_set(authority_record=other_arecord)

    def test_cannot_modify_activities(self):
        with self.new_access(self.login).cnx() as cnx:
            arecord = testutils.authority_record(cnx, name=u'a')
            cnx.commit()

            activity = arecord.reverse_used[0]

            with self.assertUnauthorized(cnx):
                activity.cw_set(description=u'hacked')

            with self.assertUnauthorized(cnx):
                activity.cw_set(associated_with=cnx.find('CWUser', login='admin').one())

            with self.assertUnauthorized(cnx):
                activity.cw_set(generated=None)

            with self.assertUnauthorized(cnx):
                activity.cw_delete()

    def test_cannot_create_activities(self):
        with self.admin_access.cnx() as cnx:
            scheme = testutils.setup_scheme(cnx, u'my scheme')
            cnx.commit()
        with self.new_access(self.login).cnx() as cnx:
            scheme = cnx.entity_from_eid(scheme.eid)
            concept = scheme.add_concept(u'lab3')
            profile = testutils.setup_profile(cnx)
            cnx.commit()

            for entity in (scheme, concept, profile):
                with self.assertUnauthorized(cnx):
                    cnx.create_entity('Activity', generated=entity)
                with self.assertUnauthorized(cnx):
                    cnx.create_entity('Activity', used=entity)

            with self.assertUnauthorized(cnx):
                cnx.create_entity('Activity', associated_with=cnx.user)

    def test_cannot_create_update_organization(self):
        with self.new_access(self.login).cnx() as cnx:
            with self.assertUnauthorized(cnx):
                testutils.authority_with_naa(cnx, u'new')

            org = testutils.authority_with_naa(cnx)
            with self.assertUnauthorized(cnx):
                org.cw_set(name=u'uh')
            with self.assertUnauthorized(cnx):
                arecord = testutils.authority_record(cnx, name=u'a', kind=u'authority')
                org.cw_set(authority_record=arecord)

    def test_cannot_create_update_naa(self):
        with self.new_access(self.login).cnx() as cnx:
            with self.assertUnauthorized(cnx):
                cnx.create_entity('ArkNameAssigningAuthority',
                                  who=u'123', what=u'443')

            test_naa = testutils.naa(cnx)
            with self.assertUnauthorized(cnx):
                test_naa.cw_set(who=u'1')

    def test_can_create_authorityrecord_activities(self):
        with self.new_access(self.login).cnx() as cnx:
            arecord = testutils.authority_record(cnx, name=u'a')
            cnx.commit()
            # EAC import expect user may create activity
            cnx.create_entity('Activity', generated=arecord)
            cnx.commit()


class ManagerUserTC(CubicWebTC):
    """Tests checking that a user in "managers" group only can do things.

    Most of the times, we do not call any assertion method and only rely on no
    error being raised.
    """
    configcls = PostgresApptestConfiguration

    assertUnauthorized = testutils.assertUnauthorized

    def test_create_update_organization(self):
        with self.admin_access.cnx() as cnx:
            org = testutils.authority_with_naa(cnx)
            cnx.commit()
            org.cw_set(name=u'uh')
            cnx.commit()
            arecord = testutils.authority_record(cnx, name=u'a', kind=u'authority')
            org.cw_set(authority_record=arecord)
            cnx.commit()

    def test_create_update_naa(self):
        with self.admin_access.cnx() as cnx:
            test_naa = testutils.naa(cnx)
            cnx.commit()
            test_naa.cw_set(who=u'1')
            cnx.commit()
            naa = cnx.create_entity('ArkNameAssigningAuthority',
                                    who=u'123', what=u'443')
            cnx.commit()
            naa.cw_set(what=u'987')
            cnx.commit()

    def test_authority_type(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_user(cnx, login=u'user', groups=('users',))
            cnx.commit()
        with self.new_access('user').client_cnx() as cnx:
            with self.assertUnauthorized(cnx):
                testutils.authority_with_naa(cnx, name=u'dream team')

    def test_authority_relation(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_user(cnx, login=u'user', groups=('users',),
                             authority=testutils.authority_with_naa(cnx))
            agent = testutils.agent(cnx, u'user')
            authority = testutils.authority_with_naa(cnx, name=u'dream team')
            cnx.commit()
            # even manager can't change an agent's authority
            with self.assertUnauthorized(cnx):
                agent.cw_set(authority=authority.eid)
        with self.new_access('user').client_cnx() as cnx:
            agent = cnx.entity_from_eid(agent.eid)
            # user can't change its own authority
            with self.assertUnauthorized(cnx):
                agent.cw_set(authority=authority.eid)
            # user can't create an agent in another authority than its own
            with self.assertUnauthorized(cnx):
                testutils.agent(cnx, u'new agent', authority=authority.eid)
            # though he can add an agent to its own authority
            testutils.agent(cnx, u'new agent')
            cnx.commit()

    def test_agent_user(self):
        with self.admin_access.repo_cnx() as cnx:
            user1 = self.create_user(cnx, login=u'user1', groups=('users',),
                                     authority=testutils.authority_with_naa(cnx))
            user2 = self.create_user(cnx, login=u'user2', groups=('users',),
                                     authority=testutils.authority_with_naa(cnx))
            agent = testutils.agent(cnx, u'user1', agent_user=user1)
            cnx.commit()
        with self.new_access('user1').client_cnx() as cnx:
            agent = cnx.entity_from_eid(agent.eid)
            # user can't change its own user
            with self.assertUnauthorized(cnx):
                agent.cw_set(agent_user=user2.eid)
            with self.assertUnauthorized(cnx):
                agent.cw_set(agent_user=None)
            # user can't create an agent and specify its associated user
            with self.assertUnauthorized(cnx):
                testutils.agent(cnx, u'user2', agent_user=user2.eid)
            agent2 = testutils.agent(cnx, u'user2')
            cnx.commit()
            with self.assertUnauthorized(cnx):
                agent2.cw_set(agent_user=user2.eid)

    def test_authority_record_base(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_user(cnx, login=u'toto', groups=('users', 'guests'),
                             authority=testutils.authority_with_naa(cnx))
            cnx.commit()
        with self.new_access('toto').client_cnx() as cnx:
            function = cnx.create_entity('AgentFunction', name=u'grouillot')
            testutils.authority_record(cnx, u'bob', reverse_function_agent=function)
            cnx.commit()
        with self.admin_access.repo_cnx() as cnx:
            cnx.execute('DELETE U in_group G WHERE U login "toto", G name "users"')
            cnx.commit()
        with self.new_access('toto').client_cnx() as cnx:
            agent = cnx.find('AuthorityRecord', has_text=u'bob').one()
            # guest user can't modify an authority record
            with self.assertUnauthorized(cnx):
                agent.cw_set(record_id=u'bobby')

    def test_authority_record_wf_permissions(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_user(cnx, login=u'toto', groups=('users', 'guests'),
                             authority=testutils.authority_with_naa(cnx))
            cnx.commit()
        with self.new_access('toto').client_cnx() as cnx:
            function = cnx.create_entity('AgentFunction', name=u'grouillot')
            record = testutils.authority_record(cnx, u'bob', reverse_function_agent=function)
            cnx.commit()
            iwf = record.cw_adapt_to('IWorkflowable')
            iwf.fire_transition('publish')
            cnx.commit()
            # we can still modify a published record
            record.reverse_name_entry_for[0].cw_set(parts=u'bobby')
            function.cw_set(name=u'director')
            cnx.commit()

    def test_update_root_badgroup(self):
        with self.admin_access.repo_cnx() as cnx:
            self.create_user(cnx, login=u'toto', groups=('users', 'guests'))
            cnx.commit()
        with self.new_access('toto').client_cnx() as cnx:
            testutils.setup_profile(cnx, title=u'pp')
            cnx.commit()
        with self.admin_access.repo_cnx() as cnx:
            cnx.execute(
                'DELETE U in_group G WHERE U login "toto", G name "users"')
            cnx.commit()
        with self.new_access('toto').client_cnx() as cnx:
            profile = cnx.find('SEDAArchiveTransfer', title=u'pp').one()
            with self.assertUnauthorized(cnx):
                profile.cw_set(title=u'qq')

    def test_sedaprofile_wf_permissions(self):
        with self.admin_access.repo_cnx() as cnx:
            profile = testutils.setup_profile(cnx, title=u'pp')
            cnx.commit()
            # Profile in draft, modifications are allowed.
            profile.cw_set(title=u'ugh')
            cnx.commit()
            comment = cnx.create_entity('SEDAComment', seda_comment=profile)
            cnx.commit()
            # Profile published, no modification allowed.
            profile.cw_adapt_to('IWorkflowable').fire_transition('publish')
            cnx.commit()
            with self.assertUnauthorized(cnx):
                profile.cw_set(title=u'huugh')
            with self.assertUnauthorized(cnx):
                comment.cw_delete()
            with self.assertUnauthorized(cnx):
                cnx.create_entity('SEDAArchivalAgreement', seda_archival_agreement=profile)

    def test_conceptscheme_wf_permissions(self):
        with self.admin_access.cnx() as cnx:
            scheme = cnx.create_entity('ConceptScheme', ark_naa=testutils.naa(cnx))
            cnx.commit()
            # in draft, modifications are allowed.
            concept = scheme.add_concept(u'blah')
            cnx.commit()
            # published, can't modify existing content.
            scheme.cw_adapt_to('IWorkflowable').fire_transition('publish')
            cnx.commit()
            with self.assertUnauthorized(cnx):
                scheme.cw_set(description=u'plop')
            with self.assertUnauthorized(cnx):
                concept.preferred_label[0].cw_set(label=u'plop')
            # though addition of new concepts / labels is fine
            new_concept = scheme.add_concept(u'plop')
            cnx.commit()
            new_label = cnx.create_entity('Label', label=u'arhg', label_of=concept)
            cnx.commit()
            # while deletion is fine for label but not for concept nor scheme
            new_label.cw_delete()
            cnx.commit()
            with self.assertUnauthorized(cnx):
                scheme.cw_delete()
            with self.assertUnauthorized(cnx):
                new_concept.cw_delete()


if __name__ == '__main__':
    import unittest
    unittest.main()



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref views related to SEDA"""

from cubicweb.web.views import uicfg

from cubicweb_seda.views import (
    CONTENT_ETYPE,
    archivetransfer,
    archiveunit,
    rtags_from_rtype_role_targets,
    simplified
)


afs = uicfg.autoform_section
pvs = uicfg.primaryview_section

# we want only simplified_profile, so its default is set to true and it only has to be hidden
afs.tag_attribute(('SEDAArchiveTransfer', 'simplified_profile'), 'main', 'hidden')
# also hide transferring and archival agency
for rtype in ('seda_transferring_agency', 'seda_archival_agency'):
    afs.tag_subject_of(('SEDAArchiveTransfer', rtype, '*'), 'main', 'hidden')
    pvs.tag_subject_of(('SEDAArchiveTransfer', rtype, '*'), 'hidden')

pvs.tag_object_of(('*', 'use_profile', '*'), 'hidden')
afs.tag_object_of(('*', 'use_profile', '*'), 'main', 'hidden')

pvs.tag_attribute(('SEDABinaryDataObject', 'filename'), 'hidden')
afs.tag_attribute(('SEDABinaryDataObject', 'filename'), 'main', 'hidden')


archivetransfer.ArchiveTransferTabbedPrimaryView.tabs.append('saem.lifecycle_tab')


# Hide "identifiant pour le service versant" which should be automatically
# filled.
class SimplifiedContentMainView(archiveunit.SimplifiedContentMainView):

    rtype_role_targets = archiveunit.SimplifiedContentMainView.rtype_role_targets[:]
    rtype_role_targets.remove(
        ('seda_transferring_agency_archive_unit_identifier', 'object', None)
    )
    rsection, display_ctrl = rtags_from_rtype_role_targets(CONTENT_ETYPE, rtype_role_targets)


simplified.simplified_afs.tag_object_of(
    ('*', 'seda_transferring_agency_archive_unit_identifier', CONTENT_ETYPE),
    'main', 'hidden',
)


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, [SimplifiedContentMainView])
    vreg.register_and_replace(SimplifiedContentMainView, archiveunit.SimplifiedContentMainView)



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref Web Services"""

from rql import TypeResolverException

from cubicweb import NoResultError
from cubicweb.view import View
from cubicweb.predicates import (match_form_params, match_http_method,
                                 match_user_groups)
from cubicweb.web.views import json

from cubicweb_saem_ref import permanent_url


class AssignArkWebService(json.JsonMixIn, View):
    """JSON view to assign a new Ark for external usage."""

    __regid__ = 'saem.ws.ark'
    __select__ = (
        match_user_groups('users', 'managers')
        & match_form_params('organization')
        & match_http_method('POST')
    )

    # XXX could check Accept=application/json
    def call(self):
        org_ark = self._cw.form['organization']
        org_ark = org_ark.replace('ark:/', '')

        def error(msg):
            self.wdata([{'error': msg.format(org_ark)}])

        try:
            org = self._cw.find('Organization', ark=org_ark).one()
        except (TypeResolverException, NoResultError):
            error('No organization matching identifier "{0}".')
        else:
            if not org.ark_naa:
                error('Organization "{0}" cannot assign ARK identifiers.')
            else:
                ark = self._cw.call_service(
                    'saem.attribute-ark', naa=org.ark_naa[0])
                self.wdata([{'ark': ark}])


class AssignArkWebServiceMissingOrganization(AssignArkWebService):
    __select__ = (
        match_user_groups('users', 'managers')
        & match_http_method('POST')
    )

    def call(self):
        self.wdata([{'error': 'Missing required "organization" query parameter.'}])


class AssignArkWebServiceNonPOST(AssignArkWebService):
    __select__ = match_user_groups('users', 'managers')

    def call(self):
        self.wdata([{'error': 'This service is only accessible using POST.'}])


class AssignArkWebServiceNonAuthenticated(AssignArkWebService):
    __select__ = ~match_user_groups('users', 'managers')

    def call(self):
        self.wdata([{'error': 'This service requires authentication.'}])


class ListArchivalUnitsWebService(json.JsonMixIn, View):
    """JSON view returning the list of archival units (site archives) with their
    associated organization units with the 'deposit' role (services versant).
    """

    __regid__ = 'saem.ws.archival-units'

    def call(self):
        def base_descr(unit):
            return {
                'name': unit.name,
                'ark': unit.ark,
                'url': permanent_url(unit),
            }

        def unit_descr(unit):
            descr = base_descr(unit)
            descr.update({
                'state': unit.cw_adapt_to('IWorkflowable').state,
                'authority': base_descr(unit.authority[0]),
            })
            return descr

        unit_rql = ('Any X,XA,XN,XSN,XO,XON,XOA WHERE '
                    'X archival_role AR, '
                    'X ark XA, X name XN, '
                    'X in_state XS, XS name XSN, '
                    'X authority XO, XO name XON, XO ark XOA')
        descriptions = []
        for archival_unit in self._cw.execute(
                unit_rql + ', AR name "archival"').entities():
            archival_unit_descr = unit_descr(archival_unit)
            archival_unit_descr['deposit-units'] = []

            for deposit_unit in self._cw.execute(
                    unit_rql + ', AR name "deposit", XO archival_unit AU, AU eid %(au)s',
                    {'au': archival_unit.eid}).entities():
                archival_unit_descr['deposit-units'].append(unit_descr(deposit_unit))

            descriptions.append(archival_unit_descr)

        self.wdata(descriptions)



for etype in ('ChronologicalRelation', 'HierarchicalRelation',
              'AssociationRelation', 'ConceptScheme',
              'use_profile', 'related_concept_scheme'):
    sync_schema_props_perms(etype)



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""Tests for agent entities"""

import datetime

from six import text_type

import mock

from cubicweb.devtools import PostgresApptestConfiguration
from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_skos.rdfio import RDFLibRDFGraph, RDFRegistry

from cubicweb_saem_ref import permanent_url
from cubicweb_saem_ref.entities import rdf

import testutils


def setUpModule():
    testutils.startpgcluster(__file__)


class RDFExportTC(testutils.XmlTestMixin, CubicWebTC):
    """Test case for RDF export"""

    configcls = PostgresApptestConfiguration

    def setUp(self):
        super(RDFExportTC, self).setUp()
        self.reg = RDFRegistry()
        self.reg.register_prefix('rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#')
        self.reg.register_prefix('dc', 'http://purl.org/dc/elements/1.1/')
        self.reg.register_prefix('dcterms', 'http://purl.org/dc/terms/')
        self.reg.register_prefix('foaf', 'http://xmlns.com/foaf/0.1/')
        self.reg.register_prefix('lglb', 'http://www.logilab.org/saem/0#')
        self.reg.register_prefix('org', 'http://www.w3.org/ns/org#')
        self.reg.register_prefix('schema', 'http://schema.org/')
        self.reg.register_prefix('vcard', 'http://www.w3.org/2006/vcard/ns#')
        self.reg.register_prefix('prov', 'http://www.w3.org/ns/prov#')

    def _rdf_triples(self, entity, adapter_name='RDFPrimary'):
        graph = RDFLibRDFGraph()
        entity.cw_adapt_to(adapter_name).fill(graph)
        return list(graph.triples())

    def assertItemsIn(self, items, container):
        """Check that elements of `items` are in `container`."""
        for item in items:
            self.assertIn(item, container)

    def assertItemsNotIn(self, items, container):
        """Check that elements of `items` are not in `container`."""
        for item in items:
            self.assertNotIn(item, container)

    def assertHasActivity(self, entity, activity, triples):
        entity_url = permanent_url(entity)
        activity_url = permanent_url(activity)
        user_url = permanent_url(activity.associated_with[0])
        self.assertItemsIn([
            (activity_url, self.uri('rdf:type'), self.uri('prov:Activity')),
            (activity_url, self.uri('prov:used'), entity_url),
            (activity_url, self.uri('prov:generated'), entity_url),
            (activity_url, self.uri('prov:wasAssociatedWith'), user_url),
            (activity_url, self.uri('prov:label'), activity.description),
            (activity_url, self.uri('prov:type'), activity.type),
            (activity_url, self.uri('prov:startedAtTime'), activity.start),
            (activity_url, self.uri('prov:endedAtTime'), activity.end),
        ], triples)

    def uri(self, s):
        return text_type(self.reg.normalize_uri(s))

    def test_organization_unit(self):
        with self.admin_access.client_cnx() as cnx:
            agent = testutils.agent(cnx, u'John Doe')
            ou = testutils.organization_unit(cnx, u'Acme Inc.',
                                             archival_roles=['deposit', 'control'],
                                             contact_point=agent)
            org = ou.authority[0]
            triples = self._rdf_triples(ou)
            self.assertItemsIn([
                (permanent_url(ou), self.uri('rdf:type'), self.uri('org:OrganizationUnit')),
                (permanent_url(ou), self.uri('dc:title'), u'Acme Inc.'),
                (permanent_url(ou), self.uri('dc:identifier'), 'ark:/' + ou.ark),
                (permanent_url(ou), self.uri('vcard:role'), u'deposit'),
                (permanent_url(ou), self.uri('vcard:role'), u'control'),
                (permanent_url(ou), self.uri('schema:contactPoint'), permanent_url(agent)),
                (permanent_url(agent), self.uri('rdf:type'), self.uri('foaf:Person')),
                (permanent_url(agent), self.uri('foaf:name'), u'John Doe'),
                (permanent_url(agent), self.uri('dc:identifier'), 'ark:/' + agent.ark),
                (permanent_url(ou), self.uri('org:unitOf'), permanent_url(org)),
            ], triples)

    def test_organization(self):
        self.reg.register_prefix('saem', 'http://www.logilab.org/saem/0#')
        with self.admin_access.client_cnx() as cnx:
            other_org = cnx.create_entity(
                'Organization', name=u'Friends of default authority',
                ark_naa=testutils.naa(cnx),
            )
            org = testutils.authority_with_naa(cnx)
            ou = testutils.organization_unit(cnx, u'archivists',
                                             archival_roles=[u'archival'],
                                             authority=other_org)
            org.cw_set(archival_unit=ou)
            org.cw_clear_all_caches()
            assert org.archival_unit[0].eid == ou.eid
            assert org.archival_authority[0].eid == other_org.eid
            triples = self._rdf_triples(org)
            expected = [
                (permanent_url(org), self.uri('rdf:type'), self.uri('org:Organization')),
                (permanent_url(org), self.uri('dc:title'), u'Default authority'),
                (permanent_url(org), self.uri('dc:identifier'), 'ark:/' + org.ark),
                (permanent_url(org), self.uri('saem:archivalUnit'), permanent_url(ou)),
                (permanent_url(org), self.uri('saem:archivalAuthority'), permanent_url(other_org)),
            ]
            self.assertItemsIn(expected, triples)

    def test_agent(self):
        with self.admin_access.client_cnx() as cnx:
            agent = testutils.agent(cnx, u'Roger Rabbit')
            triples = self._rdf_triples(agent)
            self.assertItemsIn([
                (permanent_url(agent), self.uri('rdf:type'), self.uri('foaf:Person')),
                (permanent_url(agent), self.uri('foaf:name'), u'Roger Rabbit'),
                (permanent_url(agent), self.uri('dc:identifier'), 'ark:/' + agent.ark),
            ], triples)

    def test_authority_record_rdf_base(self):
        with self.admin_access.client_cnx() as cnx:
            for kind, rdfkind in ((u'authority', 'org:OrganizationUnit'),
                                  (u'person', 'foaf:Person'),
                                  (u'family', 'foaf:Group')):
                with self.subTest(kind=kind, rdfkind=rdfkind):
                    record = testutils.authority_record(cnx, u'Acme', kind=kind)
                    cnx.commit()

                    triples = self._rdf_triples(record)
                    self.assertItemsIn([
                        (permanent_url(record), self.uri('rdf:type'), self.uri(rdfkind)),
                        (permanent_url(record), self.uri('foaf:name'), u'Acme'),
                        (permanent_url(record), self.uri('dc:identifier'), 'ark:/' + record.ark),
                    ], triples)
                    self.assertHasActivity(record, record.reverse_generated[0], triples)

    def test_authority_record_places(self):
        with self.admin_access.client_cnx() as cnx:
            record = testutils.authority_record(cnx, u'Acme Inc. Authority')
            work_address = cnx.create_entity('PostalAddress', street=u"1 av. de l'europe",
                                             postalcode=u'31400', city=u'Toulouse')
            cnx.create_entity('AgentPlace', role=u'work', place_agent=record,
                              place_address=work_address)
            home_address = cnx.create_entity('PostalAddress', street=u"Place du Capitole",
                                             postalcode=u'31000', city=u'Toulouse')
            cnx.create_entity('AgentPlace', role=u'home', place_agent=record,
                              place_address=home_address)
            triples = self._rdf_triples(record)
            self.assertItemsIn([
                (permanent_url(record), self.uri('vcard:hasAddress'), permanent_url(work_address)),
                (permanent_url(work_address), self.uri('vcard:role'), 'work'),
                (permanent_url(record), self.uri('vcard:hasAddress'), permanent_url(home_address)),
                (permanent_url(home_address), self.uri('vcard:role'), 'home'),
            ], triples)

    def test_authority_record_with_chronological_relation(self):
        with self.admin_access.client_cnx() as cnx:
            record1 = testutils.authority_record(cnx, u'Acme Inc. Authority')
            record2 = testutils.authority_record(cnx, u'Acme2 Inc. Authority')
            record3 = testutils.authority_record(cnx, u'Acme3 Inc. Authority')
            cnx.create_entity('ChronologicalRelation', chronological_predecessor=record1,
                              chronological_successor=record2)
            cnx.create_entity('ChronologicalRelation', chronological_predecessor=record2,
                              chronological_successor=record3)
            triples = self._rdf_triples(record2)
            self.assertItemsIn([
                (permanent_url(record2), self.uri('dcterms:isReplacedBy'), permanent_url(record3)),
                (permanent_url(record2), self.uri('dcterms:replaces'), permanent_url(record1)),
            ], triples)

    def test_authority_record_with_hierarchical_relation_rdf_export(self):
        with self.admin_access.client_cnx() as cnx:
            record1 = testutils.authority_record(cnx, u'Acme Inc. Authority')
            record2 = testutils.authority_record(cnx, u'Acme Group. Authority')
            cnx.create_entity('HierarchicalRelation',
                              hierarchical_parent=record2,
                              hierarchical_child=record1,
                              start_date=datetime.date(2008, 1, 1),
                              end_date=datetime.date(2099, 1, 1))
            relation_url = permanent_url(record1.reverse_hierarchical_child[0])
            time_interval_url = relation_url + '#timeInterval'
            expected_hierarchical_relations = [
                (relation_url, self.uri('org:organization'), permanent_url(record2)),
                (relation_url, self.uri('org:member'), permanent_url(record1)),
                (relation_url, self.uri('org:role'),
                 'http://www.logilab.org/saem/hierarchical_role'),
                (relation_url, self.uri('org:memberDuring'), time_interval_url),
                (time_interval_url, self.uri('schema:startDate'), datetime.date(2008, 1, 1)),
                (time_interval_url, self.uri('schema:endDate'), datetime.date(2099, 1, 1)),
            ]
            triples = self._rdf_triples(record1)
            self.assertItemsIn(expected_hierarchical_relations, triples)
            triples = self._rdf_triples(record2)
            self.assertItemsIn(expected_hierarchical_relations, triples)

    def test_authority_record_with_associative_relation_rdf_export(self):
        with self.admin_access.client_cnx() as cnx:
            record1 = testutils.authority_record(cnx, u'Acme Inc. Authority')
            record2 = cnx.create_entity('ExternalUri', uri=u'agent2', cwuri=u'agent2')
            cnx.create_entity('AssociationRelation',
                              association_from=record1,
                              association_to=record2)
            relation_url = permanent_url(record1.reverse_association_from[0])
            triples = self._rdf_triples(record1)
            self.assertItemsIn([
                (relation_url, self.uri('org:organization'), permanent_url(record1)),
                (relation_url, self.uri('org:member'), permanent_url(record2)),
                (relation_url, self.uri('org:member'), permanent_url(record1)),
                (relation_url, self.uri('org:organization'), permanent_url(record2)),
                (relation_url, self.uri('org:role'),
                 u'http://www.logilab.org/saem/association_role'),
            ], triples)

    def test_conceptscheme_rdf_prov(self):
        with self.admin_access.client_cnx() as cnx:
            scheme = testutils.setup_scheme(cnx, u'some vocab')
            cnx.commit()
            concept = scheme.add_concept(u'some concept')
            cnx.commit()

        # get scheme and concept from a fresh connection to avoid unexpected cache side effects
        # interference (where cw_clear_all_caches is not enough)
        with self.admin_access.client_cnx() as cnx:
            scheme = cnx.entity_from_eid(scheme.eid)
            concept = cnx.entity_from_eid(concept.eid)

            orig_warm_caches = rdf.ConceptSchemeRDFPrimaryAdapter.warm_caches

            mock_execute = mock.MagicMock()

            def warm_caches(self):
                orig_warm_caches(self)
                # ensure no further RQL query is done past this point by hacking .execute
                self._cw.execute = mock_execute

            with mock.patch('cubicweb_saem_ref.entities.rdf.ConceptSchemeRDFPrimaryAdapter'
                            '.warm_caches', autospec=True, side_effect=warm_caches):
                triples = self._rdf_triples(scheme)

            # scheme has two activities: its creation and addition of a concept
            self.assertHasActivity(scheme, scheme.reverse_generated[0], triples)
            self.assertHasActivity(scheme, scheme.reverse_generated[1], triples)
            self.assertHasActivity(concept, concept.reverse_generated[0], triples)
            self.assertEqual(mock_execute.call_count, 0)


if __name__ == '__main__':
    import unittest
    unittest.main()



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from io import BytesIO

from pyramid.view import view_config

from cubicweb.dataimport.importer import SimpleImportLog

from cubicweb_skos import to_unicode
from cubicweb_eac import dataimport


@view_config(
    route_name='eac.import',
    request_method='POST',
    accept='application/json',
    renderer='json',
)
def eac_import(request):
    cnx = request.cw_cnx
    if cnx.session.anonymous_session:
        request.response.status_int = 401
        return {'error': 'This service requires authentication.'}
    if cnx.user.naa is None:
        request.response.status_int = 403
        return {'error': 'Authenticated user is not linked to an organisation, '
                'or his organisation has no ARK naming authority.'}

    try:
        _, _, eid = cnx.call_service('eac.import',
                                     stream=BytesIO(request.body),
                                     import_log=SimpleImportLog('<ws>'),
                                     naa=cnx.user.naa)

    except dataimport.InvalidXML as exc:
        request.response.status_int = 400
        return {'error': 'Invalid XML file',
                'details': to_unicode(exc)}

    except dataimport.MissingTag as exc:
        if exc.tag_parent:
            err = request.cw_request.__(
                'Missing tag %(tag)s within element %(parent)s in XML file')
            params = {'tag': exc.tag, 'parent': exc.tag_parent}
        else:
            err = request.cw_request.__('Missing tag %(tag)s in XML file')
            params = {'tag': exc.tag}
        request.response.status_int = 400
        return {'error': 'Unexpected EAC data',
                'details': err % params}

    except Exception as exc:  # pylint: disable=broad-except
        cnx.exception('error while importing EAC using web service')
        request.response.status_int = 500
        return {'error': 'EAC import failed'}

    else:
        record = cnx.find('AuthorityRecord', eid=eid).one()
        return {'ark': record.ark}


def includeme(config):
    config.add_route(
        'eac.import', '/authorityrecord',
        request_method='POST',
        accept='application/json',
        # XXX it would be nice if POST with other Content-Type would be routed to cubicweb
        # header='Content-Type: application/xml',
    )
    config.scan(__name__)



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref custom views for skos entities"""


from cubicweb import tags, _
from cubicweb.view import AnyRsetView
from cubicweb.predicates import is_instance
from cubicweb.web import formwidgets as fw
from cubicweb.web.views import baseviews, treeview, uicfg

from cubicweb_skos import views as skos

from . import ImportEntityComponent


afs = uicfg.autoform_section
affk = uicfg.autoform_field_kwargs

for etype, attr in (('ConceptScheme', 'ark'),
                    ('Concept', 'ark')):
    affk.set_field_kwargs(etype, attr, widget=fw.TextInput({'size': 80}))
    affk.set_field_kwargs(etype, attr, required=False)


_('Import ConceptScheme')  # generate message used by the import component


# Have "in_scheme" inserted in attributes section so that it appears in
# cw_edited and ARK generation hook works. (This overrides uicfg tag in
# cubicweb-skos.)
afs.tag_subject_of(('Concept', 'in_scheme', '*'), 'main', 'attributes')
# Though we don't want to see the field as a Concept can only be created
# through one ConceptScheme, so hide the widget.
affk.set_field_kwargs('Concept', 'in_scheme', widget=fw.HiddenInput())


class SKOSImportComponent(ImportEntityComponent):
    """Component with a link to import a concept scheme from a SKOS file."""
    __select__ = ImportEntityComponent.__select__ & is_instance('ConceptScheme')

    @property
    def import_url(self):
        return self._cw.build_url('add/skossource')


afs.tag_object_of(('*', 'related_concept_scheme', 'ConceptScheme'), 'main', 'hidden')

skos.ConceptSchemePrimaryView.tabs.append(_('saem.lifecycle_tab'))
skos.ConceptPrimaryView.tabs.append(_('saem.lifecycle_tab'))


class ConceptSchemeSameETypeListView(baseviews.SameETypeListView):
    """Override SameETypeListView to display a search action on top"""
    __select__ = baseviews.SameETypeListView.__select__ & is_instance('ConceptScheme')

    def call(self, **kwargs):
        self.search_link()
        super(ConceptSchemeSameETypeListView, self).call(**kwargs)

    def search_link(self):
        """Render a link to search for Concepts within selected ConceptSchemes"""
        rql = 'Any X WHERE X in_scheme S, S eid '
        if len(self.cw_rset) > 1:
            rql += 'IN ({0})'.format(', '.join(str(x[0]) for x in self.cw_rset.rows))
        else:
            rql += str(self.cw_rset[0][0])
        href = self._cw.build_url(rql=rql)
        title = self._cw._('search for concepts of concept schemes')
        self.w(u'<a class="btn btn-default pull-right" href="{href}">{title}</a>'.format(
            href=href, title=title))


class ConceptTreeView(treeview.TreeView):
    """`treeview` for Concept entities."""
    __select__ = treeview.TreeView.__select__ & is_instance('Concept')
    subvid = 'skos.concept-popover'


class ConceptPopoverView(AnyRsetView):
    """Popover view for Concept, to be displayed within treeview"""
    __regid__ = 'skos.concept-popover'
    __select__ = is_instance('Concept')

    def cell_call(self, row, col, **kwargs):
        entity = self.cw_rset.get_entity(row, col)
        self._cw.add_onload("$('#{eid}').popover();".format(eid=entity.eid))
        title = entity.dc_title()
        content = u'<dl>'
        for attr in ('definition', 'example'):
            if getattr(entity, attr):
                content += u'<dt>{label}</dt><dd>{value}</dd>'.format(
                    label=self._cw._(attr),
                    value=entity.printable_value(attr, format=u'text/html'))
        content += u'</dl>'
        content += tags.a(self._cw._('view'), href=entity.absolute_url())
        data = {'data-title': title,
                'data-content': content,
                'data-html': 'true',
                'data-trigger': 'focus'}
        self.w(tags.a(title, href='javascript:$.noop();', id=str(entity.eid), **data))


def registration_callback(vreg):
    from cubicweb_skos.views import ImportConceptSchemeAction
    vreg.unregister(ImportConceptSchemeAction)
    from cubicweb.web.views import cwsources
    vreg.unregister(cwsources.ManageSourcesAction)



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""Site management views (handling of Authorities and ARK name assigning authorities among others)
"""

import re

from cubicweb import _
from cubicweb.predicates import has_permission, is_instance
from cubicweb.web import component, formwidgets as fw
from cubicweb.web.views import actions, debug, urlrewrite, uicfg

from . import SAEMRefRewriter, AddEntityComponent


pvs = uicfg.primaryview_section
afs = uicfg.autoform_section
affk = uicfg.autoform_field_kwargs

pvs.tag_object_of(('*', 'ark_naa', 'ArkNameAssigningAuthority'), 'relations')
afs.tag_object_of(('*', 'ark_naa', 'ArkNameAssigningAuthority'), 'main', 'hidden')
affk.set_field_kwargs('ArkNameAssigningAuthority', 'who', widget=fw.TextInput({'size': 80}))


class ManagementAddEntityComponent(AddEntityComponent):
    __select__ = (component.CtxComponent.__select__ & has_permission('add') &
                  is_instance('Organization', 'ArkNameAssigningAuthority'))


class ARKNamingAutorithyAction(actions.ManagersAction):
    __regid__ = 'arknameassigningauthority'
    title = _('ARK name assigning authorities')
    order = 2
    category = 'manage'


# Rewrite rules to force "sameetypelist" view for these routes (even if
# there's only one entity).
SAEMRefRewriter.rules += [
    (urlrewrite.rgx('/arknameassigningauthority', re.I),
     {'vid': 'sameetypelist',
      'rql': 'Any X, XN WHERE X who XN, X is ArkNameAssigningAuthority'}),
    (urlrewrite.rgx('/organization', re.I),
     {'vid': 'sameetypelist',
      'rql': 'Any X, XN WHERE X name XN, X is Organization'}),
]


actions.SiteConfigurationAction.order = 100
debug.SiteInfoAction.order = 101



# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-saem-ref views for AuthorityRecord"""

from six import text_type

from logilab.common.date import ustrftime
from logilab.mtconverter import xml_escape

from cubicweb import tags, _
from cubicweb.utils import json_dumps, make_uid, JSString
from cubicweb.uilib import cut, js
from cubicweb.view import EntityView
from cubicweb.predicates import has_related_entities, is_instance, match_kwargs
from cubicweb.web import formfields as ff, formwidgets as fw
from cubicweb.web.views import tabs, uicfg

from cubes.relationwidget import views as rwdg

from .. import cwuri_url, user_has_authority
from . import (ImportEntityComponent, RelatedEntitiesListView, SubviewsTabView,
               RelationInfo, editlinks, external_link)
from .widgets import JQueryIncompleteDatePicker, ConceptAutoCompleteWidget


pvs = uicfg.primaryview_section
pvdc = uicfg.primaryview_display_ctrl
afs = uicfg.autoform_section
aff = uicfg.autoform_field
affk = uicfg.autoform_field_kwargs

for etype in ('AuthorityRecord',
              'AssociationRelation', 'ChronologicalRelation', 'HierarchicalRelation',
              'Mandate', 'LegalStatus', 'EACResourceRelation'):
    affk.set_field_kwargs(etype, 'start_date',
                          widget=JQueryIncompleteDatePicker(update_min='end_date'))
    affk.set_field_kwargs(etype, 'end_date',
                          widget=JQueryIncompleteDatePicker(update_max='start_date',
                                                            default_end=True))

for rtype in ('chronological_predecessor', 'chronological_successor',
              'hierarchical_parent', 'hierarchical_child',
              'association_from', 'association_to',
              'resource_relation_resource'):
    affk.tag_subject_of(('*', rtype, '*'),
                        {'widget': rwdg.RelationFacetWidget(dialog_options={'width': 800})})

afs.tag_object_of(('*', 'use_authorityrecord', 'AuthorityRecord'), 'main', 'hidden')


class EACImportComponent(ImportEntityComponent):
    """Component with a link to import an authority record from an EAC-CPF file."""
    __select__ = (ImportEntityComponent.__select__
                  & is_instance('AuthorityRecord')
                  & user_has_authority())
    _('Import AuthorityRecord')  # generate message used by the import component

    @property
    def import_url(self):
        return self._cw.build_url('view', vid='eac.import')


# Configure edition of entity types supporting vocabulary_source/equivalent_concept

class EquivalentConceptOrTextField(ff.Field):
    def __init__(self, **kwargs):
        kwargs['required'] = True
        super(EquivalentConceptOrTextField, self).__init__(**kwargs)
        self.help = _('when linked to a vocabulary, the value is enforced to the label of a '
                      'concept in this vocabulary. Remove the vocabulary source if you want to '
                      'type text freely.')

    def get_widget(self, form):
        return ConceptAutoCompleteWidget(self.name, 'vocabulary_source', optional=True)

    def has_been_modified(self, form):
        return True  # handled in process_posted below

    def process_posted(self, form):
        posted = form._cw.form
        text_val = posted.get(self.input_name(form, 'Label'), '').strip()
        equivalent_eid = posted.get(self.input_name(form), '').strip()
        equivalent_eids = set()
        if equivalent_eid:
            equivalent_eids.add(int(equivalent_eid))
        if not (text_val or equivalent_eid):
            raise ff.ProcessFormError(form._cw.__("required field"))
        entity = form.edited_entity
        if not entity.has_eid() or getattr(entity, self.name) != text_val:
            yield (ff.Field(name=self.name, role='subject', eidparam=True), text_val)
        if (not entity.has_eid() and equivalent_eids) \
           or (entity.has_eid() and
               set(x.eid for x in entity.equivalent_concept) != equivalent_eids):
            subfield = ff.Field(name='equivalent_concept', role='subject', eidparam=True)
            # register the association between the value and the field, because on entity creation,
            # process_posted will be recalled on the newly created field, and if we don't do that it
            # won't find the proper value (this is very nasty)
            form.formvalues[(subfield, form)] = equivalent_eids
            yield (subfield, equivalent_eids)


for etype, attrs in [('AgentFunction', ('name', 'description')),
                     ('AgentPlace', ('name', 'role')),
                     ('Mandate', ('term', 'description')),
                     ('LegalStatus', ('term', 'description')),
                     ('Occupation', ('term', 'description'))]:
    aff.tag_subject_of((etype, attrs[0], '*'), EquivalentConceptOrTextField)

# handled by EquivalentConceptOrTextField
afs.tag_subject_of(('*', 'equivalent_concept', '*'), 'main', 'hidden')
pvs.tag_subject_of(('*', 'equivalent_concept', '*'), 'attributes')
pvs.tag_object_of(('*', 'vocabulary_source', '*'), 'hidden')


# Autocomplete authority record's name

class TextInputCheckSimilar(fw.TextInput):
    """Search for similar authority record names in the database and display them to the user"""

    def __init__(self, *args, **kwargs):
        self.data_initfunc = kwargs.pop('data_initfunc')
        super(TextInputCheckSimilar, self).__init__(*args, **kwargs)

    def _render(self, form, field, render):
        req = form._cw
        req.add_js('cubes.saem_ref.js')
        domid = field.dom_id(form, self.suffix)
        data = self.data_initfunc(form, field)
        category = req._("Similar entities:")
        data = json_dumps([{"label": item, "category": category} for sublist in data
                           for item in sublist])
        req.add_onload(u'cw.jqNode("%s").check_similar_values({source: %s});' % (domid, data))
        return super(TextInputCheckSimilar, self)._render(form, field, render)


def get_record_names(form, field):
    """Return names of all the authority records already existing"""
    return form._cw.execute('Any P WHERE NE parts P, NE is NameEntry').rows


affk.set_field_kwargs('NameEntry', 'parts',
                      widget=TextInputCheckSimilar({'size': 80}, data_initfunc=get_record_names))


# hide record_id for now
pvs.tag_attribute(('AuthorityRecord', 'record_id'), 'hidden')
afs.tag_attribute(('AuthorityRecord', 'record_id'), 'main', 'hidden')

afs.tag_object_of(('*', 'authority_record', 'AuthorityRecord'), 'main', 'hidden')


# AuthorityRecord primary views (tabs)

class AuthorityRecordTabbedPrimaryView(tabs.TabbedPrimaryView):
    """Tabbed primary view for AuthorityRecord"""
    __select__ = tabs.TabbedPrimaryView.__select__ & is_instance('AuthorityRecord')
    tabs = [
        _('saem_authorityrecord_general_information_tab'),
        _('saem_authorityrecord_description_tab'),
        _('saem_authorityrecord_properties_tab'),
        _('saem_authorityrecord_relations_tab'),
        _('saem.lifecycle_tab'),
    ]
    default_tab = 'saem_authorityrecord_general_information_tab'


class CitationLinkView(EntityView):
    __regid__ = 'citation-link'
    __select__ = is_instance('Citation')

    def entity_call(self, entity):
        if entity.uri:
            title = entity.note or entity.uri
            desc = cut(entity.note or u'', 50)
            self.w(u'<a class="truncate" href="%s" title="%s">%s</a>' % (
                xml_escape(entity.uri), xml_escape(desc),
                xml_escape(title)))
        else:
            title = (entity.note or
                     u'{0} #{1}'.format(self._cw._('Citation'), entity.eid))
            self.w(u'<i class="truncate">%s</i>' % title)


class AuthorityRecordPrimaryTab(tabs.PrimaryTab):
    """Main tab for authority record, just with a different regid."""
    __regid__ = 'saem_authorityrecord_general_information_tab'


class TextAttributeView(EntityView):
    """View for displaying an entity's text attribute.

    This view should be call with a ``text_attr`` parameter indicating which attribute on the entity
    contains the textual information and will display this information in a ``<div>``.

    Additionally, edit buttons will be shown floating right if user has relevant permissions. And if
    ``icon_info`` parameter is ``True``, user will also see an info button redirecting to the entity
    primary view.
    """

    __regid__ = 'saem.text_attribute'
    __select__ = EntityView.__select__ & match_kwargs('text_attr')

    @editlinks(icon_info=False)
    def entity_call(self, entity, text_attr):
        self._cw.add_js(('jquery.js', 'jquery.expander.js', 'cubes.saem_ref.js'))
        self.w(tags.div(entity.printable_value(text_attr), klass='truncate'))


class AsConceptEntityView(EntityView):
    """View for displaying an entity as a SKOS concept (to which the entity is related).

    This view should be called with ``concept_rtype`` indicating the relation to ``Concept`` entity
    type. This relation should have ``1`` or ``?`` cardinality (that is, the entity should be
    related to at most one concept)

    The view will then display a link to concept URI. The link content will be the entity`s
    ``text_attr`` attribute, or the concept label as a fallback.

    Additionally, edit buttons will be shown floating right if user has relevant permissions.
    """

    __regid__ = 'saem.entity_as_concept'
    __select__ = EntityView.__select__ & match_kwargs('concept_rtype', 'text_attr')

    @editlinks(icon_info=False)
    def entity_call(self, entity, concept_rtype, text_attr, details_attr=None,
                    **kwargs):
        # Get related concept label and uri and associated scheme title and uri
        rset = entity.related(concept_rtype)
        concept = rset.one() if rset else None
        # Compute link content
        link_content = entity.printable_value(text_attr)
        if not link_content and concept:  # empty text_attr => use concept label
            link_content = xml_escape(concept.label())
        if not link_content:  # empty text_attr, no label => empty title
            link_content = u''

        if concept:
            link = external_link(link_content, cwuri_url(concept))
            self.w(u'<strong>{0}</strong>'.format(link))
        else:
            self.w(u'<strong>{0}</strong>'.format(tags.span(link_content, escapecontent=False)))
        if concept and concept.cw_etype == 'Concept':  # Could be ExternalURI
            scheme = concept.in_scheme[0]
            source_content = u'{0}: {1}'.format(
                tags.span(self._cw._('vocabulary_source')),
                external_link(scheme.title, cwuri_url(scheme)))
            self.w(tags.div(source_content, klass='small'))
        # Compute details
        details = entity.printable_value(details_attr) if details_attr else None
        if not details and concept and concept.cw_etype == 'Concept':
            # if not specified but linked to a concept with a definition, show the definition
            details = concept.printable_value('definition')
        if details:
            self._cw.add_js(('jquery.js', 'jquery.expander.js', 'cubes.saem_ref.js'))
            self.w(tags.div(details, klass='help-block truncate'))


# AuthorityRecord EAC-CPF description tab.

class AuthorityRecordDescriptionTab(SubviewsTabView):
    """Tab view gathering EAC-CPF description information of an AuthorityRecord"""
    __regid__ = 'saem_authorityrecord_description_tab'
    __select__ = EntityView.__select__ & is_instance('AuthorityRecord')
    subvids = (
        'saem.authorityrecord.other_record_id',
        'saem.authorityrecord.places',
        'saem.authorityrecord.functions',
        'saem.authorityrecord.legal_status',
        'saem.authorityrecord.mandate',
        'saem.authorityrecord.occupation',
        'saem.authorityrecord.generalcontext',
        'saem.authorityrecord.history',
        'saem.authorityrecord.structure',
    )
    relations_info = [
        ('eac_other_record_id_of', 'object'),
        ('place_agent', 'object'),
        ('function_agent', 'object'),
        ('legal_status_agent', 'object'),
        ('mandate_agent', 'object'),
        ('occupation_agent', 'object'),
        ('general_context_of', 'object'),
        ('history_agent', 'object'),
        ('structure_agent', 'object'),
    ]


class AuthorityRecordOtherRecordIdView(RelatedEntitiesListView):
    """View for EACOtherRecordId, to be display in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.other_record_id'
    rtype = 'eac_other_record_id_of'
    subvid = 'saem.authorityrecord.other_record_id.listitem'
    _('creating EACOtherRecordId (EACOtherRecordId eac_other_record_id_of '
      'AuthorityRecord %(linkto)s)')


pvs.tag_object_of(('*', 'eac_other_record_id_of', 'AuthorityRecord'), 'hidden')


class OtherRecordIdListItemView(EntityView):
    """Display an EACOtherRecordId item."""
    __regid__ = 'saem.authorityrecord.other_record_id.listitem'

    def entity_call(self, entity, **kwargs):
        # surrounding div is necessary for proper transportation through ajax (probably because of
        # an erroneous implementation in cubicweb)
        self.w(u'<div>')
        self.wdata(u'{0}'.format(entity.value))
        if entity.local_type:
            self.wdata(u' ({0})'.format(entity.local_type))
        self.w(u'</div>')


pvs.tag_object_of(('*', 'place_agent', 'AuthorityRecord'), 'hidden')


class AuthorityRecordPlaceView(RelatedEntitiesListView):
    """View for AuthorityRecordPlace, to be display in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.places'
    rtype = 'place_agent'
    subvid = 'saem.agent_place_as_concept'
    _('creating AgentPlace (AgentPlace place_agent AuthorityRecord %(linkto)s)')


class AuthorityRecordPlaceAsConceptView(EntityView):
    """View for displaying an agent place as a SKOS concept

    If the place is related to a SKOS concept, this view will display a link to the concept URI.

    Additionally, edit buttons will be shown floating right if user has relevant permissions.
    """

    __regid__ = 'saem.agent_place_as_concept'

    def entity_call(self, entity, **kwargs):
        # Output role if any
        role = entity.printable_value('role')
        if role:
            self.w(u'<strong>{0}: </strong>'.format(role))
        # Output place's name
        entity.view('saem.entity_as_concept', w=self.w,
                    concept_rtype='equivalent_concept', text_attr='name',
                    **kwargs)
        # Output address details if they exists
        address = entity.place_address[0] if entity.place_address else None
        if address:
            self._cw.view('incontext', rset=address.as_rset(), w=self.w)


pvs.tag_object_of(('*', 'function_agent', 'AuthorityRecord'), 'hidden')


class AuthorityRecordFunctionView(RelatedEntitiesListView):
    """View for AgentFunction, to be display in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.functions'
    rtype = 'function_agent'
    subvid = 'saem.entity_as_concept'
    subvid_kwargs = {'concept_rtype': 'equivalent_concept', 'text_attr': 'name',
                     'details_attr': 'description'}
    _('creating AgentFunction (AgentFunction function_agent AuthorityRecord %(linkto)s)')


pvs.tag_object_of(('*', 'mandate_agent', 'AuthorityRecord'), 'hidden')


class AuthorityRecordMandateView(RelatedEntitiesListView):
    """View for Mandate, to be displayed in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.mandate'
    rtype = 'mandate_agent'
    subvid = 'saem.entity_as_concept'
    subvid_kwargs = {'concept_rtype': 'equivalent_concept',
                     'text_attr': 'term',
                     'details_attr': 'description'}
    _('creating Mandate (Mandate mandate_agent AuthorityRecord %(linkto)s)')


pvs.tag_object_of(('*', 'occupation_agent', 'AuthorityRecord'), 'hidden')


class AuthorityRecordOccupationView(RelatedEntitiesListView):
    """View for Occupation, to be displayed in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.occupation'
    rtype = 'occupation_agent'
    subvid = 'saem.entity_as_concept'
    subvid_kwargs = {'concept_rtype': 'equivalent_concept',
                     'text_attr': 'term',
                     'details_attr': 'description'}
    _('creating Occupation (Occupation occupation_agent AuthorityRecord %(linkto)s)')


pvs.tag_object_of(('*', 'general_context_of', 'AuthorityRecord'), 'hidden')


class GeneralContextView(RelatedEntitiesListView):
    """View for GeneralContext, to be displayed in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.generalcontext'
    rtype = 'general_context_of'
    subvid = 'saem.text_attribute'
    subvid_kwargs = {'text_attr': 'content'}
    _('creating GeneralContext (GeneralContext general_context_of AuthorityRecord %(linkto)s)')


class WithCitationViewMixIn(object):
    """View mixin displaying citation information."""
    __select__ = has_related_entities('has_citation')

    def entity_call(self, entity, *args, **kwargs):
        super(WithCitationViewMixIn, self).entity_call(entity, *args, **kwargs)
        rset = entity.related('has_citation')
        if rset:
            self.w(tags.div(u' '.join([self._cw._('Citation_plural'),
                                       self._cw.view('csv', rset,
                                                     subvid='citation-link')]),
                            klass='small'))


class WithCitationAsConceptEntityView(WithCitationViewMixIn, AsConceptEntityView):
    """Extend entity_as_concept view when entity has related citations."""
    __select__ = AsConceptEntityView.__select__ & WithCitationViewMixIn.__select__


class WithCitationTextAttributeView(WithCitationViewMixIn, TextAttributeView):
    """Extend text_attribute view when entity has related citations."""
    __select__ = TextAttributeView.__select__ & WithCitationViewMixIn.__select__


pvs.tag_object_of(('*', 'legal_status_agent', 'AuthorityRecord'), 'hidden')


class AuthorityRecordLegalStatusView(RelatedEntitiesListView):
    """View for LegalStatus, to be displayed in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.legal_status'
    rtype = 'legal_status_agent'
    subvid = 'saem.entity_as_concept'
    subvid_kwargs = {'concept_rtype': 'equivalent_concept', 'text_attr': 'term',
                     'details_attr': 'description'}
    _('creating LegalStatus (LegalStatus legal_status_agent AuthorityRecord %(linkto)s)')


pvs.tag_object_of(('*', 'history_agent', 'AuthorityRecord'), 'hidden')


class AuthorityRecordHistoryView(RelatedEntitiesListView):
    """View for History, to be displayed in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.history'
    rtype = 'history_agent'
    subvid = 'saem.text_attribute'
    subvid_kwargs = {'text_attr': 'text'}
    _('creating History (History history_agent AuthorityRecord %(linkto)s)')


pvs.tag_object_of(('*', 'structure_agent', 'AuthorityRecord'), 'hidden')


class AuthorityRecordStructureView(RelatedEntitiesListView):
    """View for Structure, to be displayed in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.structure'
    rtype = 'structure_agent'
    subvid = 'saem.text_attribute'
    subvid_kwargs = {'text_attr': 'description'}
    _('creating Structure (Structure structure_agent AuthorityRecord %(linkto)s)')


# AuthorityRecord views ###############################

class SaemListItemView(EntityView):
    """List item view working with tabid keyword argument."""
    __regid__ = 'saem.listitem'

    @editlinks(icon_info=False)
    def entity_call(self, entity, **kwargs):
        entity.view('listitem', w=self.w)

# AuthorityRecord EAC-CPF "control" (a.k.a. "properties") tab.


class AuthorityRecordPropertiesTab(SubviewsTabView):
    """Tab view gathering EAC-CPF controle information of an AuthorityRecord"""
    __regid__ = 'saem_authorityrecord_properties_tab'
    __select__ = EntityView.__select__ & is_instance('AuthorityRecord')
    subvids = (
        'saem.authorityrecord.sources',
        'saem.authorityrecord.eac_resource_relation',
    )
    relations_info = [
        ('source_agent', 'object'),
        ('resource_relation_agent', 'object'),
    ]


pvs.tag_object_of(('*', 'source_agent', 'AuthorityRecord'), 'hidden')

pvdc.tag_attribute(('EACSource', 'url'), {'vid': 'urlattr'})


class AuthorityRecordEACSourceView(RelatedEntitiesListView):
    """View for EACSource, to be display in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.sources'
    rtype = 'source_agent'
    subvid = 'saem.eacsource-listitem'
    _('creating EACSource (EACSource source_agent AuthorityRecord %(linkto)s)')


class EACSourceListItemView(EntityView):
    """List item view for EACSource."""

    __regid__ = 'saem.eacsource-listitem'
    __select__ = has_related_entities('source_agent', role='subject')

    @editlinks(icon_info=False)
    def entity_call(self, entity):
        if entity.title:
            self.w(tags.h5(entity.title, klass='list-group-item-heading'))
        if entity.description:
            self.w(u'<p class="list-group-item-text">{0}</p>'.format(
                entity.printable_value('description')))
        if entity.url:
            entity.view('saem.external_link', rtype='url', w=self.w)


pvs.tag_object_of(('*', 'resource_relation_agent', 'AuthorityRecord'), 'hidden')

pvdc.tag_subject_of(('EACResourceRelation', 'resource_relation_resource', '*'),
                    {'vid': 'saem.external_link'})


class AuthorityRecordEACResourceRelationView(RelatedEntitiesListView):
    """View for EACResourceRelation, to be displayed in the context of an AuthorityRecord"""
    __regid__ = 'saem.authorityrecord.eac_resource_relation'
    rtype = 'resource_relation_agent'
    subvid = 'saem.eacresource-listitem'
    _('creating EACResourceRelation (EACResourceRelation '
      'resource_relation_agent AuthorityRecord %(linkto)s)')


class EACResourceListItemView(EntityView):
    """List item view for EACResource."""

    __regid__ = 'saem.eacresource-listitem'
    __select__ = has_related_entities('resource_relation_agent', role='subject')

    @editlinks(icon_info=False)
    def entity_call(self, entity, **kwargs):
        resource_role = entity.printable_value('resource_role')
        resource = entity.resource
        title = u' '.join([resource_role, resource.view('saem.external_link')])
        self.w(tags.h5(title, escapecontent=False,
                       klass='list-group-item-heading'))
        if entity.agent_role:
            self.w(u'<p class="list-group-item-text"><em>{0}</em> {1}</p>'.format(
                self._cw._('agent_role'), entity.printable_value('agent_role')))
        if entity.description:
            self.w(u'<p class="list-group-item-text">{0}</p>'.format(
                entity.printable_value('description')))
        if entity.start_date or entity.end_date:
            self.w(u'<p class="list-group-item-text">{0} - {1}</p>'.format(
                entity.printable_value('start_date'),
                entity.printable_value('end_date')))


# AuthorityRecord relations tab.

class AuthorityRecordRelationsTab(SubviewsTabView):
    """Tab view for AuthorityRecord relations."""
    __regid__ = 'saem_authorityrecord_relations_tab'
    __select__ = EntityView.__select__ & is_instance('AuthorityRecord')
    subvids = (
        'saem.authorityrecord.association_relations',
        'saem.authorityrecord.chronological_relation',
        'saem.authorityrecord.hierarchical-links',
    )
    relations_info = [
        RelationInfo('chronological_predecessor', 'object',
                     label=_('ChronologicalRelation (predecessor)')),
        RelationInfo('chronological_successor', 'object',
                     label=_('ChronologicalRelation (successor)')),
        RelationInfo('hierarchical_child', 'object',
                     label=_('HierarchicalRelation (child)')),
        RelationInfo('hierarchical_parent', 'object',
                     label=_('HierarchicalRelation (parent)')),
        ('association_from', 'object'),
    ]
    _('creating AssociationRelation (AssociationRelation association_from '
      'AuthorityRecord %(linkto)s)')
    _("creating ChronologicalRelation (ChronologicalRelation "
      "chronological_predecessor AuthorityRecord %(linkto)s)")
    _("creating ChronologicalRelation (ChronologicalRelation "
      "chronological_successor AuthorityRecord %(linkto)s)")
    _('creating HierarchicalRelation (HierarchicalRelation hierarchical_child '
      'AuthorityRecord %(linkto)s)')
    _('creating HierarchicalRelation (HierarchicalRelation hierarchical_parent '
      'AuthorityRecord %(linkto)s)')


pvs.tag_object_of(('*', 'association_from', 'AuthorityRecord'), 'hidden')
pvs.tag_object_of(('*', 'association_to', 'AuthorityRecord'), 'hidden')


class RelationView(EntityView):
    """Display relations as list with edit links."""
    __regid__ = 'saem.authorityrecord.relation'
    __select__ = (EntityView.__select__
                  & is_instance('AssociationRelation',
                                'ChronologicalRelation',
                                'HierarchicalRelation')
                  & match_kwargs('main_record'))

    @editlinks(icon_info=True)
    def entity_call(self, entity, main_record):
        for other_record in (entity.subject, entity.object):
            if other_record != main_record:
                self.w(other_record.view('outofcontext'))
                break
        if entity.start_date or entity.end_date:
            self.w(tags.span(u' ({0}-{1})'.format(entity.printable_value('start_date'),
                                                  entity.printable_value('end_date')),
                             klass='text-muted'))
        if entity.description:
            self.w(tags.div(entity.printable_value('description')))


class AuthorityRecordAssociationRelationView(EntityView):
    """View for association relations to be displayed in the context of an
    AuthorityRecord on either the `from` or `to` side of the relation.
    """
    __regid__ = 'saem.authorityrecord.association_relations'
    __select__ = EntityView.__select__ & (
        has_related_entities('association_from', role='object') |
        has_related_entities('association_to', role='object'))

    def entity_call(self, entity, **kwargs):
        self.w(tags.h2(self._cw._('associated with')))
        rset = self._cw.execute(
            '(Any X,XD,XSD,XED,XAF,XAT WHERE X association_from XAF, XAF eid %(eid)s, '
            'X description XD, X start_date XSD, X end_date XED, X association_to XAT) '
            'UNION (Any X,XD,XSD,XED,XAF,XAT WHERE X association_to XAT, XAT eid %(eid)s, '
            'X description XD, X start_date XSD, X end_date XED, X association_from XAF)',
            {'eid': entity.eid})
        self._cw.view('list', rset=rset, w=self.w,
                      subvid='saem.authorityrecord.relation',
                      main_record=entity,
                      __redirectpath=entity.rest_path())


pvs.tag_object_of(('*', 'chronological_predecessor', 'AuthorityRecord'), 'hidden')
pvs.tag_object_of(('*', 'chronological_successor', 'AuthorityRecord'), 'hidden')


class RelationRelatedView(EntityView):
    __regid__ = 'saem.authorityrecord.relation.related'
    __select__ = match_kwargs('entity_rtype', 'target_rtype', 'title')

    def entity_call(self, entity, entity_rtype, target_rtype, title):
        rset = self._cw.execute(
            'Any X,XD,XSD,XED,XAF,XAT WHERE X {} XAF, XAF eid %(eid)s, '
            'X description XD, X start_date XSD, X end_date XED, X {} XAT'
            .format(entity_rtype, target_rtype), {'eid': entity.eid})
        if rset:
            self.w(tags.h3(title))
            self._cw.view('list', rset=rset, w=self.w,
                          subvid='saem.authorityrecord.relation',
                          main_record=entity,
                          __redirectpath=entity.rest_path())


class ChronologicalRelationView(EntityView):
    """Timeline view with authority records involved in a ChronologicalRelation with this authority
    record entity.
    """
    __regid__ = 'saem.authorityrecord.chronological_relation'
    __select__ = EntityView.__select__ & (
        has_related_entities('chronological_successor', role='object') |
        has_related_entities('chronological_predecessor', role='object'))
    title = _('ChronologicalRelation_plural')

    def entity_call(self, entity, **kwargs):
        self.w(tags.h2(self._cw._(self.title).lower()))

        rset = self._cw.execute(
            '(Any X WHERE X eid %(eid)s) '
            'UNION (Any S WHERE S is AuthorityRecord, RP chronological_predecessor X, '
            '       RP chronological_successor S, X eid %(eid)s) '
            'UNION (Any P WHERE P is AuthorityRecord, RS chronological_successor X, '
            '       RS chronological_predecessor P, X eid %(eid)s)',
            {'eid': entity.eid})
        json_url = self._cw.build_url('view', rql=entity.as_rset().printable_rql(),
                                      vid='saem.authorityrecord-timeline-json')
        self._cw.view('vtimeline', rset=rset, w=self.w, custom_settings={'source': json_url})

        self._cw.view('saem.authorityrecord.relation.related', entity=entity, w=self.w,
                      entity_rtype='chronological_successor',
                      target_rtype='chronological_predecessor',
                      title=self._cw._('predecessors'))
        self._cw.view('saem.authorityrecord.relation.related', entity=entity, w=self.w,
                      entity_rtype='chronological_predecessor',
                      target_rtype='chronological_successor',
                      title=self._cw._('successors'))


class AuthorityRecordTimelineJsonView(EntityView):
    """JSON view for agent with chronological relations to be used with vtimeline view."""
    __regid__ = 'saem.authorityrecord-timeline-json'
    __select__ = is_instance('AuthorityRecord')
    template = False
    content_type = 'application/json'
    binary = True

    headers = {
        'headline': '',
        'type': 'default',
        'text': '',
        'asset': {},
    }

    def entity_call(self, entity):
        data = dict(self.headers)
        data['date'] = [
            self.entity_as_date(entity, tag=self._cw._('subject')),
        ]
        for successor_relation in entity.reverse_chronological_successor:
            predecessor = successor_relation.chronological_predecessor[0]
            if predecessor.cw_etype == 'ExternalUri':
                continue
            data['date'].append(self.entity_as_date(
                predecessor, tag=self._cw._('chronological_predecessor')))
        for predecessor_relation in entity.reverse_chronological_predecessor:
            successor = predecessor_relation.chronological_successor[0]
            if successor.cw_etype == 'ExternalUri':
                continue
            data['date'].append(self.entity_as_date(
                successor, tag=self._cw._('chronological_successor')))
        self.w(json_dumps({'timeline': data}))

    @staticmethod
    def entity_as_date(entity, tag):
        """Return a dict suitable for insertion within the `date` entry of
        TimelineJS JSON structure.
        """
        date = {'tag': tag}
        calendarable = entity.cw_adapt_to('ICalendarable')
        if calendarable and (calendarable.start or calendarable.stop):
            date['headline'] = entity.view('incontext')
            if calendarable.start:
                date['startDate'] = ustrftime(calendarable.start, '%Y,%m,%d')
            if calendarable.stop:
                date['endDate'] = ustrftime(calendarable.stop, '%Y,%m,%d')
        return date


def _node(entity):
    return {
        'id': text_type(entity.eid),
        'title': cut(entity.dc_title(), 30),
    }


class AuthorityRecordGraphView(EntityView):
    __select__ = EntityView.__select__ & (
        has_related_entities('hierarchical_parent', role='object') |
        has_related_entities('hierarchical_child', role='object'))

    __regid__ = 'saem.authorityrecord.hierarchical-links'
    title = _('HierarchicalRelation_plural')

    def entity_call(self, entity, **kwargs):
        self.w(tags.h2(self._cw._(self.title).lower()))

        self._cw.add_js(('jquery.js', 'jquery.jOrgChart.js'))
        self._cw.add_css('jquery.jOrgChart.css')
        hiera_list = u'<ul id="hierarchical_relations" style="display:none">'
        tags_closure = u'</ul>'
        rset_parents = self._cw.execute('Any P WHERE PR hierarchical_child X, '
                                        'PR hierarchical_parent P, X eid %(eid)s',
                                        {'eid': entity.eid})
        if rset_parents:
            parent = rset_parents.get_entity(0, 0)
            nprops = _node(parent)
            hiera_list += u'<li>{}<ul>'.format(nprops['title'])
            tags_closure = u'</ul></li>' + tags_closure
        nprops = _node(entity)
        hiera_list += u'<li class="main">{}<ul>'.format(nprops['title'])
        tags_closure = u'</ul></li>' + tags_closure
        for child_rel in entity.reverse_hierarchical_parent:
            for child in child_rel.hierarchical_child:
                nprops = _node(child)
                hiera_list += u'<li></li>'.format(nprops['title'])
        hiera_list += tags_closure
        self.w(hiera_list)
        domid = make_uid()
        self._cw.add_onload(js.cw.jqNode('hierarchical_relations').jOrgChart(
            JSString('{chartElement: %s}' % js.cw.jqNode(domid))))
        self.w(u'<div id="%s"></div>' % domid)
        if len(rset_parents) > 1:
            self.w(u'<div class="other-parents icon-attention">')
            self.w(self._cw._('AuthorityRecord %s has several parents:') % entity.dc_title())
            self._cw.view('list', rset_parents, w=self.w)
            self.w(u'</div>')

        self._cw.view('saem.authorityrecord.relation.related', entity=entity, w=self.w,
                      entity_rtype='hierarchical_child',
                      target_rtype='hierarchical_parent',
                      title=self._cw._('parents'))
        self._cw.view('saem.authorityrecord.relation.related', entity=entity, w=self.w,
                      entity_rtype='hierarchical_parent',
                      target_rtype='hierarchical_child',
                      title=self._cw._('children'))


pvs.tag_object_of(('*', 'hierarchical_parent', 'AuthorityRecord'), 'hidden')
pvs.tag_object_of(('*', 'hierarchical_child', 'AuthorityRecord'), 'hidden')
